<?php

function evercurrent_run_update_check($key = NULL) {
  $key = $key ?: _evercurrent_get_key();
  if (!$key) {
    _evercurrent_write_status(RMH_STATUS_WARNING, 'Update check not run. No key');
    return FALSE;
  }
  if ($available = update_get_available(TRUE)) {
    module_load_include('inc', 'update', 'update.compare');
    $data = update_calculate_project_data($available);
  }
  else {
    _evercurrent_write_status(RMH_STATUS_WARNING, 'Unable to get available updates');
    drupal_set_message('Unable to get available updates');
    return FALSE;
  }
  $evercurrent_version = 0;
  $projects = array();

  $projects = db_select('system', 's')
    ->fields('s', array('name', 'type', 'status', 'info'))
    ->execute()
    ->fetchAll();
  foreach ($projects as $project) {
    $project->info = unserialize($project->info);
    if ($project->name == 'evercurrent' && isset($project->info['version'])) {
      $evercurrent_version = $project->info['version'];
    }
  }

  $projects[] = array(
    'name' => 'drupal',
    'type' => 'core',
    'status' => 1,
    'info' => array(
      'core' => '7.x',
      'version' => VERSION,
      'project' => 'drupal',
      'hidden' => FALSE,
    ),
  );

  $sender_data = [
    'send_url' => variable_get('evercurrent_target_address', RMH_ENV_URL),
    'url' => _evercurrent_get_environment_url(),
    'api_key' => $key,
    'module_version' => $evercurrent_version,
    'api_version' => 3,
    'projects' => $projects,
  ];

  // Expose hook to add anything else.
  drupal_alter('evercurrent_update_data', $sender_data);

  $response = _evercurrent_send_updates($sender_data);
  return $response;
}

/**
 * Helper function.
 *
 * Retrieve a key from settings.php, or from variable.
 */
function _evercurrent_get_key() {
  $override = variable_get('evercurrent_override', FALSE);
  // Key from regular configuration
  $config_key = variable_get('evercurrent_environment_token', NULL);
  // Key from settings.php
  $settings_key = variable_get('evercurrent_key', NULL);
  return ($config_key && !$override) ? $config_key : $settings_key;
}

function _evercurrent_send_updates($sender_data) {
  $path = $sender_data['send_url'] . RMH_URL;
  $headers = array('Content-Type' => 'application/json');
  $data = drupal_json_encode($sender_data);
  $options = array(
    'headers' => $headers,
    'method' => 'POST',
    'data' => $data
  );

  $response = drupal_http_request($path, $options);

  if ($response->code != 200) {
    _evercurrent_write_status(RMH_STATUS_ERROR, 'Error code ' . $response->code . ' when trying to post to ' . $path);
  }
  else {
    // Check the response data, was it successful?
    $response_data = json_decode($response->data);
    if ($response_data) {
      $saved = $response_data->saved;
      if (!$saved) {
        _evercurrent_write_status(RMH_STATUS_ERROR, check_plain($response_data->message));
        return FALSE;
      }
      else {
        _evercurrent_write_status(RMH_STATUS_OK, $response_data->message);
        return TRUE;
      }
    }
  }
}

/**
 * Discontinues the process of sending updates.
 * Saves a status, message and last try time.
 *
 * @param $severity
 * @param $message
 */
function _evercurrent_write_status($severity, $message) {
  $message = check_plain($message);
  variable_set(RMH_STATUS, $severity);
  variable_set(RMH_STATUS_MESSAGE, $message);
  // If error, also log to watchdog.
  if ($severity == RMH_STATUS_ERROR) {
    watchdog('RMH', $message, array(), WATCHDOG_ERROR);
  }
  // If successful, set the last try timestamp
  if ($severity == RMH_STATUS_OK) {
    variable_set(RMH_LAST_TRY, time());
  }
}

/**
 * Helper function.
 *
 * Get an environment URL and ship together with the results.
 * First we see if we have our own explicit variable set. This
 * is only used for this purpose, and it allows the module
 * to be flexible in terms of determining the correct environment.
 *
 * If we do not find this variable, we fallback to the $base url.
 */
function _evercurrent_get_environment_url() {
  global $base_url;
  $env_url = variable_get('evercurrent_environment_url');
  // If env_url, send that.
  $url = $env_url ? $env_url : $base_url;
  return $url;
}
